@@ -5,11 +5,12 @@
 #' @param .trj the trajectory object.
 #' @inheritParams select
 #' @param resource the name of the resource.
-#' @param amount the amount to seize/release, accepts either a numeric or a callable object
-#' (a function) which must return a numeric.
-#' @param continue a boolean (if \code{post.seize} OR \code{reject} is defined) or a pair of booleans
-#' (if \code{post.seize} AND \code{reject} are defined) to indicate whether these subtrajectories
-#' should continue to the next activity in the main trajectory.
+#' @param amount the amount to seize/release, accepts either a numeric or a callable
+#' object (a function) which must return a numeric.
+#' @param continue a boolean (if \code{post.seize} OR \code{reject} is defined)
+#' or a pair of booleans (if \code{post.seize} AND \code{reject} are defined; if
+#' only one value is provided, it will be recycled) to indicate whether these
+#' subtrajectories should continue to the next activity in the main trajectory.
 #' @param post.seize an optional trajectory object which will be followed after a successful seize.
 #' @param reject an optional trajectory object which will be followed if the arrival is rejected.
 #'
@@ -140,7 +141,7 @@ timeout <- function(.trj, task) UseMethod("timeout")
 
 #' @export
 timeout.trajectory <- function(.trj, task) {
-  check_args(task=c("number", "function"))
+  check_args(task=c("numeric", "function"))
   .trj$timeout(task)
 }
 
@@ -192,31 +193,31 @@ set_attribute.trajectory <- function(.trj, keys, values, global=FALSE, mod=c(NA,
 set_global <- function(.trj, keys, values, mod=c(NA, "+", "*"))
   set_attribute(.trj, keys, values, TRUE, mod=mod)
 
-#' Activate/Deactivate Generators
+#' Activate/Deactivate Sources
 #'
 #' Activities for activating or deactivating the generation of arrivals by name.
 #'
 #' @inheritParams seize
-#' @param generator the name of the generator or a function returning a name.
+#' @param source the name of the source or a function returning a name.
 #'
 #' @return Returns the trajectory object.
-#' @seealso \code{\link{set_trajectory}}, \code{\link{set_distribution}}.
+#' @seealso \code{\link{set_trajectory}}, \code{\link{set_source}}.
 #' @export
-activate <- function(.trj, generator) UseMethod("activate")
+activate <- function(.trj, source) UseMethod("activate")
 
 #' @export
-activate.trajectory <- function(.trj, generator) .trj$activate(generator)
+activate.trajectory <- function(.trj, source) .trj$activate(source)
 
 #' @rdname activate
 #' @export
-deactivate <- function(.trj, generator) UseMethod("deactivate")
+deactivate <- function(.trj, source) UseMethod("deactivate")
 
 #' @export
-deactivate.trajectory <- function(.trj, generator) .trj$deactivate(generator)
+deactivate.trajectory <- function(.trj, source) .trj$deactivate(source)
 
-#' Set Generator Parameters
+#' Set Source Parameters
 #'
-#' Activities for modifying a generator's trajectory or distribution by name.
+#' Activities for modifying a source's trajectory or source object by name.
 #'
 #' @inheritParams seize
 #' @inheritParams activate
@@ -225,21 +226,30 @@ deactivate.trajectory <- function(.trj, generator) .trj$deactivate(generator)
 #' @return Returns the trajectory object.
 #' @seealso \code{\link{activate}}, \code{\link{deactivate}}.
 #' @export
-set_trajectory <- function(.trj, generator, trajectory) UseMethod("set_trajectory")
+set_trajectory <- function(.trj, source, trajectory) UseMethod("set_trajectory")
 
 #' @export
-set_trajectory.trajectory <- function(.trj, generator, trajectory)
-  .trj$set_trajectory(generator, trajectory)
+set_trajectory.trajectory <- function(.trj, source, trajectory)
+  .trj$set_trajectory(source, trajectory)
 
 #' @rdname set_trajectory
-#' @param distribution a function modelling the interarrival times (returning a
-#' negative value stops the generator).
+#' @param object a function modelling the interarrival times (if the source type
+#' is a generator; returning a negative value stops the generator) or a data frame
+#' (if the source type is a data source).
 #' @export
-set_distribution <- function(.trj, generator, distribution) UseMethod("set_distribution")
+set_source <- function(.trj, source, object) UseMethod("set_source")
 
 #' @export
-set_distribution.trajectory <- function(.trj, generator, distribution)
-  .trj$set_distribution(generator, distribution)
+set_source.trajectory <- function(.trj, source, object)
+  .trj$set_source(source, object)
+
+#' @rdname set_trajectory
+#' @export
+set_distribution <- function(.trj, source, object) UseMethod("set_distribution") # nocov
+
+#' @export
+set_distribution.trajectory <- function(.trj, source, object) # nocov
+  .Deprecated("set_source") # nocov
 
 #' Set Prioritization Values
 #'
@@ -271,9 +281,11 @@ set_prioritization.trajectory <- function(.trj, values, mod=c(NA, "+", "*"))
 #' \code{0} and \code{N}. A return value equal to \code{0} skips the branch and
 #' continues to the next activity. A returning value between \code{1} to \code{N}
 #' makes the arrival to follow the corresponding sub-trajectory.
-#' @param continue a vector of \code{N} booleans that indicate whether the arrival must
-#' continue to the main trajectory after each sub-trajectory or not.
-#' @param ... \code{N} trajectory objects describing each sub-trajectory.
+#' @param continue a vector of \code{N} booleans that indicate whether the arrival
+#' must continue to the main trajectory after each sub-trajectory or not (if only
+#' one value is provided, it will be recycled to match the number of sub-trajectories).
+#' @param ... \code{N} trajectory objects (or a list of \code{N} trajectory objects)
+#' describing each sub-trajectory.
 #'
 #' @return Returns the trajectory object.
 #' @export
@@ -357,8 +369,8 @@ renege_abort.trajectory <- function(.trj) .trj$renege_abort()
 #' @inheritParams seize
 #' @param n number of clones, accepts either a numeric or a callable object
 #' (a function) which must return a numeric.
-#' @param ... optional parallel sub-trajectories. Each clone will follow
-#' a different sub-trajectory if available.
+#' @param ... a number of optional parallel sub-trajectories (or a list of
+#' sub-trajectories). Each clone will follow a different sub-trajectory if available.
 #'
 #' @return Returns the trajectory object.
 #' @export
