@@ -14,14 +14,26 @@ bool Process::deactivate() {
   return true;
 }
 
-void Generator::set_first_activity() {
-  RFn head(trj["head"]);
-  first_activity = Rcpp::as<Rcpp::XPtr<Activity> >(head());
+Activity* trj_head(const REnv& trj) { return trj_get(trj, "head"); }
+
+Arrival* Source::new_arrival(double delay) {
+  // format the name and create the next arrival
+  std::string arr_name = name + boost::lexical_cast<std::string>(count++);
+  Arrival* arrival = new Arrival(sim, arr_name, is_monitored(),
+                                 order, first_activity, count);
+
+  if (sim->verbose) Rcpp::Rcout <<
+    FMT(10, right) << sim->now() << " |" <<
+    FMT(12, right) << "source: " << FMT(15, left) << name << "|" <<
+    FMT(12, right) << "new: " << FMT(15, left) << arr_name << "| " <<
+    (sim->now() + delay) << std::endl;
+
+  return arrival;
 }
 
 void Generator::run() {
   // get the delay for the next (n) arrival(s)
-  Rcpp::NumericVector delays = dist();
+  RNum delays = source();
   size_t n = delays.size();
   double delay = 0;
 
@@ -30,16 +42,43 @@ void Generator::run() {
       return;
     delay += delays[i];
 
-    // format the name and create the next arrival
-    std::string arr_name = name + boost::lexical_cast<std::string>(count++);
-    Arrival* arrival = new Arrival(sim, arr_name, is_monitored(),
-                                   order, first_activity, count);
+    // schedule the arrival
+    sim->schedule(delay, new_arrival(delay),
+                  first_activity->priority ? first_activity->priority : count);
+  }
+  // schedule the generator
+  sim->schedule(delay, this, priority);
+}
 
-    if (sim->verbose) Rcpp::Rcout <<
-      FMT(10, right) << sim->now() << " |" <<
-      FMT(12, right) << "generator: " << FMT(15, left) << name << "|" <<
-      FMT(12, right) << "new: " << FMT(15, left) << arr_name << "| " <<
-      (sim->now() + delay) << std::endl;
+void DataSrc::run() {
+  double delay = 0;
+  RNum col, time = source[col_time];
+  int i = 0;
+
+  while (i++ != batch) {
+    if (time.size() <= count)
+      return;
+    delay += time[count];
+
+    Arrival* arrival = new_arrival(delay);
+
+    for (size_t j = 0; j < col_attrs.size(); ++j) {
+      col = source[col_attrs[j]];
+      arrival->set_attribute(col_attrs[j], col[count-1]);
+    }
+
+    if (col_priority) {
+      col = source[*col_priority];
+      arrival->order.set_priority(col[count-1]);
+    }
+    if (col_preemptible) {
+      col = source[*col_preemptible];
+      arrival->order.set_preemptible(col[count-1]);
+    }
+    if (col_restart) {
+      col = source[*col_restart];
+      arrival->order.set_restart(col[count-1]);
+    }
 
     // schedule the arrival
     sim->schedule(delay, arrival,
@@ -158,7 +197,7 @@ void Arrival::terminate(bool finished) {
   }
   unset_remaining();
   if (is_monitored() >= 1 && !dynamic_cast<Batched*>(this))
-    sim->record_end(name, lifetime.start, lifetime.activity, finished);
+    sim->mon->record_end(name, lifetime.start, sim->now(), lifetime.activity, finished);
   delete this;
 }
 
@@ -166,7 +205,7 @@ void Arrival::set_attribute(const std::string& key, double value, bool global) {
   if (global) return sim->set_attribute(key, value);
   attributes[key] = value;
   if (is_monitored() >= 2)
-    sim->record_attribute(name, key, value);
+    sim->mon->record_attribute(sim->now(), name, key, value);
 }
 
 double Arrival::get_attribute(const std::string& key, bool global) const {
@@ -236,11 +275,11 @@ void Arrival::renege(Activity* next) {
 
 void Arrival::report(const std::string& resource) const {
   ArrTime time = restime.find(resource)->second;
-  sim->record_release(name, time.start, time.activity, resource);
+  sim->mon->record_release(name, time.start, sim->now(), time.activity, resource);
 }
 
 void Arrival::report(const std::string& resource, double start, double activity) const {
-  sim->record_release(name, start, activity, resource);
+  sim->mon->record_release(name, start, sim->now(), activity, resource);
 }
 
 bool Arrival::leave_resources(bool flag) {
