@@ -32,7 +32,7 @@ class Simulator {
 
   typedef MSET<Event> PQueue;
   typedef UMAP<Process*, PQueue::iterator> EvMap;
-  typedef UMAP<std::string, Entity*> EntMap;
+  typedef MAP<std::string, Entity*> EntMap;
   typedef UMAP<Arrival*, USET<std::string> > ArrMap;
   typedef UMAP<std::string, Batched*> NamBMap;
   typedef UMAP<Activity*, Batched*> UnnBMap;
@@ -43,14 +43,16 @@ class Simulator {
 public:
   std::string name;
   bool verbose;
+  Monitor* mon;
 
   /**
    * Constructor.
    * @param name    simulator name
    * @param verbose verbose flag
+   * @param mon     monitoring object
    */
-  Simulator(const std::string& name, bool verbose)
-    : name(name), verbose(verbose), now_(0), process_(NULL), b_count(0) {}
+  Simulator(const std::string& name, bool verbose, Monitor* mon)
+    : name(name), verbose(verbose), mon(mon), now_(0), process_(NULL), b_count(0) {}
 
   ~Simulator() {
     foreach_ (EntMap::value_type& itr, resource_map)
@@ -90,10 +92,7 @@ public:
     b_count = 0;
     signal_map.clear();
     attributes.clear();
-    mon_arr_traj.clear();
-    mon_arr_res.clear();
-    mon_attributes.clear();
-    mon_resources.clear();
+    mon->clear();
   }
 
   double now() const { return now_; }
@@ -122,7 +121,7 @@ public:
   /**
    * Look for future events.
    */
-  Rcpp::DataFrame peek(int steps) const {
+  RData peek(int steps) const {
     VEC<double> time;
     VEC<std::string> process;
     if (steps) {
@@ -132,49 +131,36 @@ public:
         if (!--steps) break;
       }
     }
-    return Rcpp::DataFrame::create(
+    return RData::create(
       Rcpp::Named("time")             = time,
       Rcpp::Named("process")          = process,
       Rcpp::Named("stringsAsFactors") = false
     );
   }
 
-  /**
-   * Process the next event. Only one step, a giant leap for mankind.
-   */
-  bool step(double until = -1) {
-    if (event_queue.empty())
-      return false;
-    PQueue::iterator ev = event_queue.begin();
-    if (until >= 0 && until <= ev->time) {
-      if (until > now_)
-        now_ = until;
-      return false;
-    }
-    now_ = ev->time;
-    process_ = ev->process;
-    event_map.erase(ev->process);
-    process_->run();
-    process_ = NULL;
-    event_queue.erase(ev);
-    return true;
-  }
-
   /**
    * Executes steps until the given criterion is met.
    * @param   until   time of ending
    */
   void run(double until) {
     size_t nsteps = 0;
-    while (step(until))
+    while (_step(until))
       if (++nsteps % 100000 == 0) Rcpp::checkUserInterrupt();
+    mon->flush();
+  }
+
+  void step(unsigned int n = 1) {
+    size_t nsteps = 0;
+    while (n-- && _step())
+      if (++nsteps % 100000 == 0) Rcpp::checkUserInterrupt();
+    mon->flush();
   }
 
   /**
    * Add a generator of arrivals to the simulator.
    * @param   name_prefix     prefix for the arrival names
    * @param   trj             a user-defined R trajectory
-   * @param   dis             a user-defined R function that provides random numbers
+   * @param   dist            a user-defined R function that provides random numbers
    * @param   mon             monitoring level
    * @param   priority        arrival priority
    * @param   preemptible     maximum priority that cannot cause preemption (>=priority)
@@ -194,6 +180,35 @@ public:
     return true;
   }
 
+  /**
+   * Attach arrivals from a data frame.
+   * @param   name_prefix     prefix for the arrival names
+   * @param   trj             a user-defined R trajectory
+   * @param   data            a user-supplied data frame
+   * @param   mon             monitoring level
+   * @param   time            column name
+   * @param   attrs           column names for attributes
+   * @param   priority        column name
+   * @param   preemptible     column name
+   * @param   restart         column name
+   */
+  bool add_dataframe(const std::string& name_prefix, REnv trj, RData data, int mon,
+                     int batch, const std::string& time, const VEC<std::string>& attrs,
+                     const OPT<std::string>& priority,
+                     const OPT<std::string>& preemptible,
+                     const OPT<std::string>& restart)
+  {
+    if (process_map.find(name_prefix) != process_map.end()) {
+      Rcpp::warning("process '%s' already defined", name_prefix);
+      return false;
+    }
+    DataSrc* gen = new DataSrc(this, name_prefix, mon, trj, data, batch, time,
+                               attrs, priority, preemptible, restart);
+    process_map[name_prefix] = gen;
+    gen->activate();
+    return true;
+  }
+
   /**
    * Add a resource to the simulator.
    * @param   name              the name
@@ -257,13 +272,13 @@ public:
   }
 
   /**
-   * Get a generator by name.
+   * Get a source by name.
    */
-  Generator* get_generator(const std::string& name) const {
+  Source* get_source(const std::string& name) const {
     EntMap::const_iterator search = process_map.find(name);
     if (search == process_map.end())
-      Rcpp::stop("generator '%s' not found (typo?)", name);
-    return static_cast<Generator*>(search->second);
+      Rcpp::stop("source '%s' not found (typo?)", name);
+    return static_cast<Source*>(search->second);
   }
 
   /**
@@ -283,6 +298,21 @@ public:
     return arrival;
   }
 
+  void record_ongoing(bool per_resource) const {
+    foreach_ (const ArrMap::value_type& itr1, arrival_map) {
+      if (dynamic_cast<Batched*>(itr1.first) || !itr1.first->is_monitored())
+        continue;
+      if (!per_resource)
+        mon->record_end(itr1.first->name, itr1.first->get_start(), R_NaReal, R_NaReal, false);
+      else foreach_ (const EntMap::value_type& itr2, resource_map) {
+        double start = itr1.first->get_start(itr2.second->name);
+        if (start < 0)
+          continue;
+        mon->record_release(itr1.first->name, start, R_NaReal, R_NaReal, itr2.second->name);
+      }
+    }
+  }
+
   Batched** get_batch(Activity* ptr, const std::string& id) {
     if (id.size()) {
       if (namedb_map.find(id) == namedb_map.end())
@@ -334,7 +364,7 @@ public:
 
   void set_attribute(const std::string& key, double value) {
     attributes[key] = value;
-    record_attribute("", key, value);
+    mon->record_attribute(now_, "", key, value);
   }
   double get_attribute(const std::string& key) const {
     Attr::const_iterator search = attributes.find(key);
@@ -350,121 +380,6 @@ public:
     arrival_map.erase(arrival);
   }
 
-  /**
-   * Record monitoring data.
-   */
-  void record_end(const std::string& name, double start, double activity, bool finished) {
-    mon_arr_traj.insert("name",           name);
-    mon_arr_traj.insert("start_time",     start);
-    mon_arr_traj.insert("end_time",       now_);
-    mon_arr_traj.insert("activity_time",  activity);
-    mon_arr_traj.insert("finished",       finished);
-  }
-  void record_release(const std::string& name, double start, double activity, const std::string& resource) {
-    mon_arr_res.insert("name",            name);
-    mon_arr_res.insert("start_time",      start);
-    mon_arr_res.insert("end_time",        now_);
-    mon_arr_res.insert("activity_time",   activity);
-    mon_arr_res.insert("resource",        resource);
-  }
-  void record_attribute(const std::string& name, const std::string& key, double value) {
-    mon_attributes.insert("time",         now_);
-    mon_attributes.insert("name",         name);
-    mon_attributes.insert("key",          key);
-    mon_attributes.insert("value",        value);
-  }
-  void record_resource(const std::string& name, int server_count, int queue_count,
-                       int capacity, int queue_size) {
-    mon_resources.insert("resource",      name);
-    mon_resources.insert("time",          now_);
-    mon_resources.insert("server",        server_count);
-    mon_resources.insert("queue",         queue_count);
-    mon_resources.insert("capacity",      capacity);
-    mon_resources.insert("queue_size",    queue_size);
-  }
-
-  /**
-   * Get monitoring data.
-   */
-  Rcpp::DataFrame get_mon_arrivals(bool per_resource, bool ongoing) const {
-    if (!per_resource) {
-      VEC<std::string> name             = mon_arr_traj.get<std::string>("name");
-      VEC<double> start_time            = mon_arr_traj.get<double>("start_time");
-      VEC<double> end_time              = mon_arr_traj.get<double>("end_time");
-      VEC<double> activity_time         = mon_arr_traj.get<double>("activity_time");
-      Rcpp::LogicalVector finished      = Rcpp::wrap(mon_arr_traj.get<bool>("finished"));
-      if (ongoing) {
-        foreach_ (const ArrMap::value_type& itr, arrival_map) {
-          if (dynamic_cast<Batched*>(itr.first) || !itr.first->is_monitored())
-            continue;
-          name.push_back(itr.first->name);
-          start_time.push_back(itr.first->get_start());
-          end_time.push_back(R_NaReal);
-          activity_time.push_back(R_NaReal);
-          finished.push_back(R_NaInt);
-        }
-      }
-      return Rcpp::DataFrame::create(
-        Rcpp::Named("name")             = name,
-        Rcpp::Named("start_time")       = start_time,
-        Rcpp::Named("end_time")         = end_time,
-        Rcpp::Named("activity_time")    = activity_time,
-        Rcpp::Named("finished")         = finished,
-        Rcpp::Named("stringsAsFactors") = false
-      );
-    } else {
-      VEC<std::string> name             = mon_arr_res.get<std::string>("name");
-      VEC<double> start_time            = mon_arr_res.get<double>("start_time");
-      VEC<double> end_time              = mon_arr_res.get<double>("end_time");
-      VEC<double> activity_time         = mon_arr_res.get<double>("activity_time");
-      VEC<std::string> resource         = mon_arr_res.get<std::string>("resource");
-      if (ongoing) {
-        foreach_ (const ArrMap::value_type& itr1, arrival_map) {
-          if (dynamic_cast<Batched*>(itr1.first) || !itr1.first->is_monitored())
-            continue;
-          foreach_ (const EntMap::value_type& itr2, resource_map) {
-            double start = itr1.first->get_start(itr2.second->name);
-            if (start < 0)
-              continue;
-            name.push_back(itr1.first->name);
-            start_time.push_back(start);
-            end_time.push_back(R_NaReal);
-            activity_time.push_back(R_NaReal);
-            resource.push_back(itr2.second->name);
-          }
-        }
-      }
-      return Rcpp::DataFrame::create(
-        Rcpp::Named("name")             = name,
-        Rcpp::Named("start_time")       = start_time,
-        Rcpp::Named("end_time")         = end_time,
-        Rcpp::Named("activity_time")    = activity_time,
-        Rcpp::Named("resource")         = resource,
-        Rcpp::Named("stringsAsFactors") = false
-      );
-    }
-  }
-  Rcpp::DataFrame get_mon_attributes() const {
-    return Rcpp::DataFrame::create(
-      Rcpp::Named("time")             = mon_attributes.get<double>("time"),
-      Rcpp::Named("name")             = mon_attributes.get<std::string>("name"),
-      Rcpp::Named("key")              = mon_attributes.get<std::string>("key"),
-      Rcpp::Named("value")            = mon_attributes.get<double>("value"),
-      Rcpp::Named("stringsAsFactors") = false
-    );
-  }
-  Rcpp::DataFrame get_mon_resources() const {
-    return Rcpp::DataFrame::create(
-      Rcpp::Named("resource")         = mon_resources.get<std::string>("resource"),
-      Rcpp::Named("time")             = mon_resources.get<double>("time"),
-      Rcpp::Named("server")           = mon_resources.get<int>("server"),
-      Rcpp::Named("queue")            = mon_resources.get<int>("queue"),
-      Rcpp::Named("capacity")         = mon_resources.get<int>("capacity"),
-      Rcpp::Named("queue_size")       = mon_resources.get<int>("queue_size"),
-      Rcpp::Named("stringsAsFactors") = false
-    );
-  }
-
 private:
   double now_;              /**< simulation time */
   Process* process_;        /**< running process */
@@ -478,10 +393,27 @@ private:
   size_t b_count;           /**< unnamed batch counter */
   SigMap signal_map;        /**< map of arrivals subscribed to signals */
   Attr attributes;          /**< user-defined (key, value) pairs */
-  Monitor mon_arr_traj;     /**< arrival statistics per trajectory */
-  Monitor mon_arr_res;      /**< arrival statistics per resource */
-  Monitor mon_attributes;   /**< attribute statistics */
-  Monitor mon_resources;    /**< resource statistics */
+
+  /**
+   * Process the next event. Only one step, a giant leap for mankind.
+   */
+  bool _step(double until = -1) {
+    if (event_queue.empty())
+      return false;
+    PQueue::iterator ev = event_queue.begin();
+    if (until >= 0 && until <= ev->time) {
+      if (until > now_)
+        now_ = until;
+      return false;
+    }
+    now_ = ev->time;
+    process_ = ev->process;
+    event_map.erase(ev->process);
+    process_->run();
+    process_ = NULL;
+    event_queue.erase(ev);
+    return true;
+  }
 };
 
 #endif
