@@ -8,7 +8,7 @@ test_that("a schedule name conflicts with a generator name", {
 })
 
 test_that("a schedule cannot be created if the corresponding resource doesn't exist", {
-  ptr <- Simulator__new("dummy", FALSE)
+  ptr <- simmer()$.__enclos_env__$private$sim_obj
   expect_error(add_resource_manager_(ptr, "name", "capacity", c(0, 1), c(0, 1), -1))
 })
 
